"""Constants for the Roku integration."""
DOMAIN = "roku"

# Attributes
ATTR_IDENTIFIERS = "identifiers"
ATTR_KEYWORD = "keyword"
ATTR_MANUFACTURER = "manufacturer"
ATTR_MODEL = "model"
ATTR_SOFTWARE_VERSION = "sw_version"
ATTR_SUGGESTED_AREA = "suggested_area"

# Default Values
DEFAULT_PORT = 8060

# Services
SERVICE_SEARCH = "search"
