"""Constants for the kmtronic integration."""

DOMAIN = "kmtronic"

CONF_HOSTNAME = "host"
CONF_USERNAME = "username"
CONF_PASSWORD = "password"

DATA_HUB = "hub"
DATA_HOST = "host"
DATA_COORDINATOR = "coordinator"

MANUFACTURER = "KMtronic"
ATTR_MANUFACTURER = "manufacturer"
ATTR_IDENTIFIERS = "identifiers"
ATTR_NAME = "name"
